/*
 * This file is part of the Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2018 Nicolas Darnis <ndarnis@free.fr>.
 *
 * GDSL is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GDSL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GDSL.  If not, see <http://www.gnu.org/licenses/>. 
 */

/*
 * GDSL - Generic Data Structures Library
 * $RCSfile: gdsl_macros.h,v $
 * $Revision: 1.14 $
 * $Date: 2015/02/17 12:22:57 $
 */


#ifndef _GDSL_MACROS_H_
#define _GDSL_MACROS_H_


#ifdef __cplusplus
extern "C" 
{
#endif /* __cplusplus */

/**
 * @addtogroup gdsl_macros Various macros module.
 * @brief This module provides some various macros.
 * @{
 */

#ifdef GDSL_MAX
#undef GDSL_MAX
#endif

/**
 * @brief Give the greatest number of two numbers.
 *
 * @note Complexity: O( 1 )
 * @pre X & Y must be basic scalar C types
 * @param X First scalar variable
 * @param Y Second scalar variable
 * @return X if X is greather than Y.
 * @return Y if Y is greather than X.
 * @see GDSL_MIN()
 */
#define GDSL_MAX(X,Y) (X>Y?X:Y)

#ifdef GDSL_MIN
#undef GDSL_MIN
#endif

/**
 * @brief Give the lowest number of two numbers
 *
 * @note Complexity: O( 1 )
 * @pre X & Y must be basic scalar C types
 * @param X First scalar variable
 * @param Y Second scalar variable
 * @return Y if Y is lower than X.
 * @return X if X is lower than Y.
 * @see GDSL_MAX()
 */
#define GDSL_MIN(X,Y) (X>Y?Y:X)


/**
 * @}
 */


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _GDSL_MACROS_H_ */


/** EMACS **
 * Local variables:
 * mode: c
 * c-basic-offset: 4
 * End:
 */
